@@ -1,5 +1,25 @@
 package SQL::Translator::Producer::Latex;
 
+# -------------------------------------------------------------------
+# $Id: Latex.pm,v 1.1 2006/03/21 20:07:21 cmungall Exp $
+# -------------------------------------------------------------------
+# Copyright (C) 2002-6 SQLFairy Authors
+#
+# This program is free software; you can redistribute it and/or
+# modify it under the terms of the GNU General Public License as
+# published by the Free Software Foundation; version 2.
+#
+# This program is distributed in the hope that it will be useful, but
+# WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+# General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
+# 02111-1307  USA
+# -------------------------------------------------------------------
+
 =pod
 
 =head1 NAME
@@ -34,11 +54,12 @@ automatically arrange them horizontally or vertically.
 
 =cut
 
-use strict;
-use warnings;
+# -------------------------------------------------------------------
 
-our @EXPORT_OK;
-our $VERSION = '1.59';
+use strict;
+
+use vars qw[ $DEBUG $VERSION @EXPORT_OK ];
+$VERSION = sprintf "%d.%02d", q$Revision: 1.1 $ =~ /(\d+)\.(\d+)/;
 $DEBUG   = 0 unless defined $DEBUG;
 
 use SQL::Translator::Utils 'debug';
@@ -84,14 +105,16 @@ sub latex {
     $s =~ s/([\&\_\$\{\#])/\\$1/g;
     return $s;
 }
-
+ 
 1;
 
+# -------------------------------------------------------------------
+
 =pod
 
 =head1 AUTHOR
 
-Chris Mungall
+Chris Mungall 
 
 =head1 SEE ALSO
 
